package atom

import (
	"errors"
	"fmt"
	"net/mail"
)

type Person struct {
	*CommonAttributes
	Name       string              `xml:"name"`
	URI        string              `xml:"uri,omitempty"`   // IRI
	Email      string              `xml:"email,omitempty"` // EmailAddress
	Extensions []*ExtensionElement `xml:",any,omitempty"`
}

// NewPerson creates a new Person. It returns a *Person and an error.
func NewPerson(name string) (*Person, error) {
	if name == "" {
		return nil, errors.New("error creating new person: name string empty")
	}

	return &Person{Name: name}, nil
}

// SetURI sets the URI element of the Person. It returns an error.
func (l *Link) SetURI(uri string) error {
	if !isValidIRI(uri) {
		return fmt.Errorf("uri %v not correctly formatted", uri)
	}

	return nil
}

// AddExtension adds the Extension to the Person. It returns an error.
func (p *Person) AddExtension(e *ExtensionElement) error {
	if e == nil {
		return errors.New("error adding extension element to person: *ExtensionElement is nil")
	}

	if p.Extensions == nil {
		p.Extensions = make([]*ExtensionElement, 1)
		p.Extensions[0] = e
	} else {
		p.Extensions = append(p.Extensions, e)
	}

	return nil
}

// Check checks the Person for incompatibilities with RFC4287. It returns an
// error.
func (p *Person) Check() error {
	if p.Name == "" {
		return fmt.Errorf("name element of person %v empty", p)
	}

	if p.URI != "" {
		if !isValidIRI(p.URI) {
			return fmt.Errorf("uri element of person %v not correctly formatted", p)
		}
	}

	if p.Email != "" {
		if _, err := mail.ParseAddress(p.Email); err != nil {
			return fmt.Errorf("email element of person %v not correctly formatted", p)
		}
	}

	if p.Extensions != nil {
		for i, e := range p.Extensions {
			if err := e.Check(); err != nil {
				return fmt.Errorf("extension element %v of person %v: %v", i, p, err)
			}
		}
	}

	return nil
}
